/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * Test initiation implementation with a single reference of multiplicity 1..1
 * @author Bryan Aupperle
 *
 */

#include "TestInterface1.h"
#include "SCA.h"

/* @Property */
char testName[50];

/* @Reference(name="reference1", interfaceHeader="Service1.h", required="false") */

struct TestException testFault;

char *invokeTest(const char *input) {
	SCAREF serviceToken;
	int compCode, reason, size = 50, num_values = 1;
	char *pTestName = testName, response1[255], faultTemp[25], *result;
	wchar_t faultName[25], *pFaultName = faultName;

	SCAPropertyCString(L"testName", &pTestName, &size, &num_values, &compCode, &reason);

	result = (char *)malloc(255);  /* infrastructure must free this */

	SCAGetReference(L"reference1", &serviceToken, &compCode, &reason);
	if (serviceToken) {
		size = 255;
		SCAInvoke(serviceToken, L"operation1", strlen(input), (void *)input,
	              &size, (void *)response1, &compCode, &reason);

		switch (compCode) {
			case (SCACC_FAULT):
				size = 255;
				SCAGetFaultMessage(serviceToken, &size, &pFaultName, (void *)response1, &compCode, &reason);

				strcpy(result, "Test service got a fault during execution: ");
				wcstombs(faultTemp, faultName, 25);
				strcat(result, faultTemp);
				strcat(result, " - ");
				strcat(result, response1);
				testFault.faultInfo = result;
				SCASetFaultMessage(L"", L"", L"TestException", sizeof(testFault), (void *)&testFault,
						           &compCode, &reason);

				free(result);
				return "";

			case (SCACC_ERROR):
				strcpy(result, "Test service got an error during execution: ");
				switch (reason) {
					case (SCA_SERVICE_UNAVAILABLE):
						strcat(result, "Service unavailable");
						break;
					case (SCA_MULTIPLE_SERVICES):
						strcat(result, "Multiple services");
						break;
					case (SCA_PARAMETER_ERROR):
						strcat(result, "Incorrect parameter value");
						break;
					default:
						strcat(result, "Other error");
				}
				testFault.faultInfo = result;
				SCASetFaultMessage(L"", L"", L"TestException", sizeof(testFault), (void *)&testFault,
						           &compCode, &reason);

				free(result);
				return "";

			case (SCACC_OK):
				strcpy(result, testName);
				strcat(result, " ");
				strcat(result, input);
				strcat(result, " ");
				strcat(result, response1);
				break;

			default:
				/* Should not get here */
				result[0] = '\0';
		}
	}
	else {
		strcpy(result, testName);
		strcat(result, " ");
		strcat(result, input);
		strcat(result, "  no invocation");
	}

	return result;
};
