/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */

/**
 * C component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service1
 * reference gets called service1 operation1 is invoked
 *
 * @author Bryan Aupperle
 *
 */

#include "Service1.h"
#include "SCA.h"

/* @Property */
char serviceName[50];

/* @Reference(name="reference1", interfaceHeader="Service11.h") */

char *operation1(const char *input) {
	SCAREF serviceToken;
	int compCode, reason, size = 50, num_values = 1;
	char *pServiceName = serviceName, response1[255], *result;

	SCAPropertyCString(L"serviceName", &pServiceName, &size, &num_values, &compCode, &reason);
	
	result = (char *)malloc(255);  /* infrastructure must free this */

	SCAGetReference(L"reference1", &serviceToken, &compCode, &reason);
	size = 255;
	SCAInvoke(serviceToken, L"operation1", strlen(input), (void *)input,
	          &size, (void *)response1, &compCode, &reason);
	switch (compCode) {
		case (SCACC_ERROR):
			strcpy(result, "Test service got an error during execution: ");
			switch (reason) {
				case (SCA_PARAMETER_ERROR):
					strcat(result, "Incorrect parameter value");
					break;
				case (SCA_RUNTIME_ERROR):
					strcat(result, "Other runtime detected error");
					break;
				default:
					strcat(result, "Other error");
			}
			break;

		case (SCACC_OK):
			strcpy(result, serviceName);
			strcat(result, " operation1 invoked ");
			strcat(result, response1);
			break;

		default:
			/* Should not get here */
			result[0] = '\0';
	}

	return result;
};
