/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * C component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service1
 * Two threads are created and reference gets called on each thread when
 * service1 operation1 is invoked
 * @author Bryan Aupperle
 *
 */


#include "Service1.h"
#include "SCA.h"
#include <pthread.h>

/* @Property */
char serviceName[50];
SCAREF serviceToken;
char responseT[255];

/* @Reference(name="reference1", interfaceHeader="Service1.h") */


void *thread1(void *data) {
	int compCode, reason, size = 255;

	SCAInvoke(serviceToken, L"operation1", 2, (void *)"a", &size, (void *)responseT,
			  &compCode, &reason);

	pthread_exit((void *)responseT);
};

void *thread2(void *data) {
	int compCode, reason, size = 255;
	char response1[255];

	SCAInvoke(serviceToken, L"operation1", 2, (void *)"b", &size, (void *)response1,
			  &compCode, &reason);

	pthread_exit(NULL);
};

char *operation1(const char *input) {
	int compCode, reason, size = 50, num_values = 1;
	char *pServiceName = serviceName, *result;
	pthread_t tid1, tid2;
	void *response1;

	SCAPropertyCString(L"serviceName", &pServiceName, &size, &num_values, &compCode, &reason);

	result = (char *)malloc(255);  /* infrastructure must free this */

	SCAGetReference(L"reference1", &serviceToken, &compCode, &reason);
	pthread_create(&tid1, NULL, thread1, NULL);
	pthread_create(&tid2, NULL, thread2, NULL);

	pthread_join(tid1, &response1);
	pthread_join(tid2, NULL);

	strcpy(result, serviceName);
	strcat(result, " operation1 invoked ");
	strcat(result, response1);
	return result;
};
